# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from .....schema_classes import AssertionEvaluationContextClass
from .....schema_classes import AssertionEvaluationParametersClass
from .....schema_classes import AssertionEvaluationParametersTypeClass
from .....schema_classes import AssertionEvaluationSpecClass
from .....schema_classes import AssertionMonitorClass
from .....schema_classes import AssertionMonitorBootstrapConfigClass
from .....schema_classes import AssertionMonitorBootstrapStatusClass
from .....schema_classes import AssertionMonitorCapabilityClass
from .....schema_classes import AssertionMonitorMetricsCubeBootstrapStateClass
from .....schema_classes import AssertionMonitorMetricsCubeBootstrapStatusClass
from .....schema_classes import AssertionMonitorSettingsClass
from .....schema_classes import AssertionTimeBucketingStrategyClass
from .....schema_classes import AuditLogSpecClass
from .....schema_classes import DataHubOperationSpecClass
from .....schema_classes import DatasetFieldAssertionParametersClass
from .....schema_classes import DatasetFieldAssertionSourceTypeClass
from .....schema_classes import DatasetFreshnessAssertionParametersClass
from .....schema_classes import DatasetFreshnessSourceTypeClass
from .....schema_classes import DatasetSchemaAssertionParametersClass
from .....schema_classes import DatasetSchemaSourceTypeClass
from .....schema_classes import DatasetVolumeAssertionParametersClass
from .....schema_classes import DatasetVolumeSourceTypeClass
from .....schema_classes import EmbeddedAssertionClass
from .....schema_classes import MonitorErrorClass
from .....schema_classes import MonitorErrorTypeClass
from .....schema_classes import MonitorInfoClass
from .....schema_classes import MonitorModeClass
from .....schema_classes import MonitorStateClass
from .....schema_classes import MonitorStatusClass
from .....schema_classes import MonitorSuiteAssertionSettingsClass
from .....schema_classes import MonitorSuiteAssignmentSpecClass
from .....schema_classes import MonitorSuiteInfoClass
from .....schema_classes import MonitorSuiteNotificationSettingsClass
from .....schema_classes import MonitorSuiteTypeClass
from .....schema_classes import MonitorTimeseriesStateClass
from .....schema_classes import MonitorTypeClass


AssertionEvaluationContext = AssertionEvaluationContextClass
AssertionEvaluationParameters = AssertionEvaluationParametersClass
AssertionEvaluationParametersType = AssertionEvaluationParametersTypeClass
AssertionEvaluationSpec = AssertionEvaluationSpecClass
AssertionMonitor = AssertionMonitorClass
AssertionMonitorBootstrapConfig = AssertionMonitorBootstrapConfigClass
AssertionMonitorBootstrapStatus = AssertionMonitorBootstrapStatusClass
AssertionMonitorCapability = AssertionMonitorCapabilityClass
AssertionMonitorMetricsCubeBootstrapState = AssertionMonitorMetricsCubeBootstrapStateClass
AssertionMonitorMetricsCubeBootstrapStatus = AssertionMonitorMetricsCubeBootstrapStatusClass
AssertionMonitorSettings = AssertionMonitorSettingsClass
AssertionTimeBucketingStrategy = AssertionTimeBucketingStrategyClass
AuditLogSpec = AuditLogSpecClass
DataHubOperationSpec = DataHubOperationSpecClass
DatasetFieldAssertionParameters = DatasetFieldAssertionParametersClass
DatasetFieldAssertionSourceType = DatasetFieldAssertionSourceTypeClass
DatasetFreshnessAssertionParameters = DatasetFreshnessAssertionParametersClass
DatasetFreshnessSourceType = DatasetFreshnessSourceTypeClass
DatasetSchemaAssertionParameters = DatasetSchemaAssertionParametersClass
DatasetSchemaSourceType = DatasetSchemaSourceTypeClass
DatasetVolumeAssertionParameters = DatasetVolumeAssertionParametersClass
DatasetVolumeSourceType = DatasetVolumeSourceTypeClass
EmbeddedAssertion = EmbeddedAssertionClass
MonitorError = MonitorErrorClass
MonitorErrorType = MonitorErrorTypeClass
MonitorInfo = MonitorInfoClass
MonitorMode = MonitorModeClass
MonitorState = MonitorStateClass
MonitorStatus = MonitorStatusClass
MonitorSuiteAssertionSettings = MonitorSuiteAssertionSettingsClass
MonitorSuiteAssignmentSpec = MonitorSuiteAssignmentSpecClass
MonitorSuiteInfo = MonitorSuiteInfoClass
MonitorSuiteNotificationSettings = MonitorSuiteNotificationSettingsClass
MonitorSuiteType = MonitorSuiteTypeClass
MonitorTimeseriesState = MonitorTimeseriesStateClass
MonitorType = MonitorTypeClass

# fmt: on
