{
  "type": "record",
  "Aspect": {
    "name": "recommendationModuleKey",
    "keyForEntity": "recommendationModule",
    "entityCategory": "_unset_",
    "entityAspects": [
      "recommendationModule"
    ]
  },
  "name": "RecommendationModuleKey",
  "namespace": "com.linkedin.pegasus2avro.metadata.key",
  "fields": [
    {
      "type": "string",
      "name": "moduleId",
      "doc": "Module ID for the recommendation module"
    },
    {
      "type": "string",
      "name": "identifier",
      "doc": "Identifier for the specific module"
    }
  ],
  "doc": "Key for a recommendation module"
}