{
  "type": "record",
  "Aspect": {
    "name": "monitorKey",
    "keyForEntity": "monitor",
    "entityCategory": "core",
    "entityAspects": [
      "monitorInfo",
      "monitorBootstrapStatus",
      "monitorTimeseriesState",
      "monitorAnomalyEvent",
      "status"
    ]
  },
  "name": "MonitorKey",
  "namespace": "com.linkedin.pegasus2avro.metadata.key",
  "fields": [
    {
      "Relationship": {
        "entityTypes": [
          "dataset"
        ],
        "name": "Monitors"
      },
      "Searchable": {
        "fieldType": "URN"
      },
      "UrnValidation": {
        "entityTypes": [
          "dataset"
        ],
        "exist": false,
        "strict": true
      },
      "java": {
        "class": "com.linkedin.pegasus2avro.common.urn.Urn"
      },
      "type": "string",
      "name": "entity",
      "doc": "The entity being monitored",
      "Urn": "Urn",
      "entityTypes": [
        "dataset"
      ]
    },
    {
      "type": "string",
      "name": "id",
      "doc": "Unique id for the monitor."
    }
  ],
  "doc": "Key for a Monitor"
}