{
  "type": "record",
  "Aspect": {
    "name": "serviceKey",
    "keyForEntity": "service",
    "entityCategory": "core",
    "entityAspects": [
      "serviceProperties",
      "mcpServerProperties",
      "subTypes",
      "ownership",
      "status",
      "globalTags",
      "dataPlatformInstance"
    ],
    "entityDoc": "A Service represents an external service that can be integrated with DataHub, such as an MCP server."
  },
  "name": "ServiceKey",
  "namespace": "com.linkedin.pegasus2avro.service",
  "fields": [
    {
      "type": "string",
      "name": "id",
      "doc": "Unique identifier for this service.\nExamples: glean-search, internal-tools, weather-api"
    }
  ],
  "doc": "Key for a Service entity.\n\nA Service represents an external service that can be integrated with DataHub,\nsuch as an MCP server, REST API, or other service types."
}