# mypy: ignore-errors
# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
# isort: skip_file
from .....schema_classes import DataHubPageTemplateAssetSummaryClass
from .....schema_classes import DataHubPageTemplatePropertiesClass
from .....schema_classes import DataHubPageTemplateRowClass
from .....schema_classes import DataHubPageTemplateSurfaceClass
from .....schema_classes import DataHubPageTemplateVisibilityClass
from .....schema_classes import PageTemplateScopeClass
from .....schema_classes import PageTemplateSurfaceTypeClass
from .....schema_classes import SummaryElementClass
from .....schema_classes import SummaryElementTypeClass


DataHubPageTemplateAssetSummary = DataHubPageTemplateAssetSummaryClass
DataHubPageTemplateProperties = DataHubPageTemplatePropertiesClass
DataHubPageTemplateRow = DataHubPageTemplateRowClass
DataHubPageTemplateSurface = DataHubPageTemplateSurfaceClass
DataHubPageTemplateVisibility = DataHubPageTemplateVisibilityClass
PageTemplateScope = PageTemplateScopeClass
PageTemplateSurfaceType = PageTemplateSurfaceTypeClass
SummaryElement = SummaryElementClass
SummaryElementType = SummaryElementTypeClass

# fmt: on
