{
  "type": "record",
  "Aspect": {
    "name": "schemafieldInfo"
  },
  "name": "SchemaFieldInfo",
  "namespace": "com.linkedin.pegasus2avro.schemafield",
  "fields": [
    {
      "Searchable": {
        "fieldNameAliases": [
          "_entityName"
        ],
        "fieldType": "KEYWORD",
        "searchLabel": "entityName",
        "searchTier": 1
      },
      "type": [
        "null",
        "string"
      ],
      "name": "name",
      "default": null
    },
    {
      "Searchable": {
        "/*": {
          "fieldType": "URN",
          "queryByDefault": true
        }
      },
      "type": [
        "null",
        {
          "type": "array",
          "items": "string"
        }
      ],
      "name": "schemaFieldAliases",
      "default": null,
      "doc": "Used to store field path variations for the schemaField urn."
    }
  ]
}